# -*- coding: utf8 -*-
from __future__ import unicode_literals

import xml.etree.cElementTree as ET

from driverng.protocol.base import BaseProtocol
from driverng.utils import (
    splitlines,
    prettify_xml
)


class AquaLinkProtocol(BaseProtocol):

    def __init__(self, multipart_astm=False, **kwargs):
        self.multipart_astm = multipart_astm
        super(AquaLinkProtocol, self).__init__(**kwargs)

    def send_trames(self, trames):
        self.write(trames)
        return True

    def build_trames(self, message):
        if message and bytearray([message[-1]]) not in (b"\r", b"\n"):
            message += b"\r\n"
        technique_response = ""
        msg = message.decode(self.encoding)  # on veut rester en unicode pour eviter "UnicodeDecodeError: 'ascii' codec .."
        if u"GetTestsResponse" in msg:
            technique_response = "http://www.ngnydevices.tech/GetTestsResponse"
        elif u"ConveyorInitializationResponse" in msg:
            technique_response = "http://www.ngnydevices.tech/ConveyorInitializationResponse"
        return b"{}\r\n{}\r\n{}\r\nSOAPAction: \"{}\"\r\n{}\r\nContent-Length: {}\r\n\r\n{}".format(
            "POST /aqualis/TestPort HTTP/1.1",
            "Accept: text/xml, multipart/related",
            "Content-Type: text/xml; charset=utf-8",
            technique_response,
            "Connection: keep-alive",
            len(message),
            message
        )

    def recv_trames(self):
        message = self.read(size=4096)
        # Résultats
        if "SendResults" in message:
            # Response message to send
            data = ""
            data = ET.Element("S:Envelope")
            data.set("xmlns:S", "http://schemas.xmlsoap.org/soap/envelope/")
            body = ET.SubElement(data, "S:Body")
            send_results = ET.SubElement(body, "SendResultsResponse")
            send_results.set("xmlns", "http://www.ngnydevices.tech/aqualis/3-0")
            ET.SubElement(send_results, "Result").text = "RESULT_RECORDED"
            data_xml = prettify_xml(ET.tostring(data, 'utf-8'))
            data_to_send = b"{}\r\n{}\r\n{}\r\nSOAPAction: \"{}\"\r\n{}\r\nContent-Length: {}\r\n\r\n{}".format(
                "POST /aqualis/ResultPort HTTP/1.1",
                "Accept: text/xml, multipart/related",
                "Content-Type: text/xml; charset=utf-8",
                "http://www.ngnydevices.tech/SendResultsResponse",
                "Connection: keep-alive",
                len(data_xml),
                data_xml
            )
            self.write(data_to_send)
        if "S:Envelope" not in message:
            return ""
        # elif "S:Envelope" in message and ("GetTests", "SendResults", "ConveyorInitialization") not in message:
        #    self.write(b"HTTP/1.1 501 Not Implemented")
        #    return False
        elif message.splitlines():
            return message.splitlines()[-1]
        else:
            return message

    @staticmethod
    def get_messages(message):
        messages = []
        current_message = []

        for line in splitlines(message):
            current_message.append(line)

        if current_message:
            messages.append(b"\n".join(current_message))

        return messages
